clear
set more off

local path    = "U:\tf\Matteo\Rshock\Replication"	    // where files for regressions are


cls
cd `path'

adopath ++ `path'\ADOFILES
set scrollbufsize 300000

use PANEL_DATASET, clear

set scheme s1color



local smpl1 "1965q1"
local smpl2 "2016q2"



local ii_val= 1 // 1-Monetary Shock; 2-Demand Shocks
local subsample_rule= 1 // 1-Estimate Taylor rule by sample
local nlags=4	 // Lags of LHS variables as controls in shock estimation 
local nlagslp=2	 // Lags of LHS variables as controls in local projection estimation
local sh = 1 	 // IRF on sh standard deviations
local lagsvv = 1 // Lags of conditioning variable in interaction term
local plag = 21  // Periods for projection

local save_dum = 0 // save_dum = 1 to save results in csv files -- for paper plots




if `ii_val' == 1{ 
local sname = "rshock" 
}
if `ii_val' == 2{ 
local sname = "yshock" 
}

if `subsample_rule' == 1{
	local addname = "samplerule"
}
else{
	local addname = ""
}



* Fragility measures to use
* 1 - High Inflation: BAD
* 2 - High CA: GOOD
* 3 - High Reserves: GOOD
* 4 - High External Debt: BAD
* 5 - High GDP PC relative to average: GOOD
* 6 - Fast Growth
* 7 - Fixed Exchange Rate
* 8 - Trade with US
* 9 - Trade with RoW
* 10 - External Debt xres
* 11 - Change in External Debt


// Times periods
#delimit ;
global T1lb = "1965q1"; global T1ub = "1985q4";
global T2lb = "1986q1"; global T2ub = "2016q4";
global Tnum = 2; 
#delimit cr

// dumies for time-sample
local dumvars = " "

forvalues tt = 1 (1) $Tnum {
	gen dum`tt' = 0
	
	local tlb = `"T`tt'lb"'
	local tub = `"T`tt'ub"'
	
	replace dum`tt' = 1 if tin($`tlb' , $`tub')
	
	local dumvars  = " `dumvars'  dum`tt' "	
}

di "`dumvars'"



levelsof quarter , c l(datelist)
foreach qq of local datelist {
	local name = string(`qq', "%tq")
	gen dum`name' = cond(quarter == `qq',1,0)
}


drop US_r_shock*

gen years_after = (trend-1)/4
label var years_after "Year Since Shock"


***************************************************
* PART 1: CONSTRUCT US MONETARY AND DEMAND SHOCKS
***************************************************

xtset country quarter, quarterly 
gen qq = quarter(dofq(quarter)) // qofd(quarter)
gen yy = year(dofq(quarter)) 
gen yq = yy + (qq-1)/4

drop yy qq

order countrylong quarter year yq 

preserve

keep if countrylong=="United States"



#delimit ;
local shockregression_controls_r = 
"L(1/`nlags').US_r 
L(0/`nlags').US_y 
L(0/`nlags').US_dp 
L(0/`nlags').US_spread 
L(0/`nlags').ln_gdp_for 
quarter quartersq" ;
#delimit cr 


if `subsample_rule' == 1 {

gen UU1 = .

forvalues tt = 1 (1) $Tnum {
	local tlb = `"T`tt'lb"'
	local tub = `"T`tt'ub"'
	
	regress US_r `shockregression_controls_r' if tin($`tlb' , $`tub') // if tin(1965q1,1985q4) 
	predict UUrE if tin($`tlb' , $`tub'), residuals 
	replace UU1 = UUrE	if tin($`tlb' , $`tub')
	drop UUrE
	}

}
else{
regress US_r `shockregression_controls_r'
predict US_r_fit
predict UU1, residuals
gen UUr = UU1
}


//L(1/`nlags').ln_gdp_for 

#delimit ;
local shockregression_controls_y = 
"L(1/`nlags').US_r 
L(1/`nlags').US_y 
L(1/`nlags').US_dp 
L(1/`nlags').ln_gdp_for 
L(1/`nlags').US_spread 
quarter quartersq" ;
#delimit cr 

regress US_y `shockregression_controls_y'
predict US_y_fit
predict UU2, residuals
gen UUy = UU2


keep quarter yq US* UU*
tempfile US_file2
save `US_file2'

if `save_dum' == 1 {
outsheet using US_file.csv, comma noquote replace
}

restore

merge m:1 quarter using `US_file2', nogen
xtset country quarter, quarterly 


sort country quarter
gen yy  = ln_gdp
gen yyr = ln_gdp_c
gen rr  = US_r
gen xr  = US_xr

tssmooth ma UU1_ma4 = UU1, window(3 1 0)
tssmooth ma UU2_ma4 = UU2, window(3 1 0)

label var UU1_ma4 "Monetary Shock, 4 qtr MA"
label var UU2_ma4 "Demand Shock, 4 qtr MA"
label var rr "FFR"
label var d4_ln_gdp_us "US GDP growth yoy, %"


gen ypanel = .
gen irfy_f = .
gen irfy_f_se = .
gen irfy_h = .
gen irfy_h_se = .


label var trend "Quarters"

**********************************************
* 1.5- DECLARE SPACE FOR IRFS COMPUTATIONS
**********************************************

gen residu = 0
gen resida = 0
gen reside = 0
gen residr = 0
gen residx = 0
gen residm = 0

forvalues tt = 1 (1) $Tnum {
	* EME
	qui gen irfy_eme_h_p`tt'    = .  
	qui gen irfy_eme_h_p`tt'_se = .  
	
	* AFE
	qui gen irfy_afe_h_p`tt'    = .  
	qui gen irfy_afe_h_p`tt'_se = .  
	
	* US
	qui gen irfy_us_h_p`tt'    = .  
	qui gen irfy_us_h_p`tt'_se = .  
	qui gen irfr_us_h_p`tt'    = .  
	qui gen irfr_us_h_p`tt'_se = .  
}

**************************************
* 2-  PANEL REGRESSIONS BEGINS HERE 
**************************************

sort country quarter

if `ii_val' == 1 {
//		local addquarters = "dum1981q4"
		local addquarters = "dum1981q4"
		local titler "1. Fed Funds Rate - ppt"
		local titley "2. U.S. GDP - %"
}
if `ii_val' == 2 {
		local addquarters = " "
		local titler = "2. Fed Funds Rate - ppt"
		local titley = "1. U.S. GDP - %"
}

// interaction term: shock x period-dumy
local Urhs = " "
forvalues tt = 1 (1) $Tnum {

	gen UUp`tt'  = UU`ii_val' * dum`tt'  // 	gen UU`vv'_f  = UU`ii_val' * (F`vv'-pclx)/(pcux) 
	
	local Urhs = "`Urhs' UUp`tt' "	
}

di "`Urhs'"


// Regression step //
forvalues x = 0/`plag' {
		

		**********************
		* R US
		**********************		
		quietly replace ypanel = F`x'.rr-L1.rr
		qui replace ypanel=. if dum_us!=1
		di "lag US" `x'
		
		#delimit ;
		qui xtreg ypanel `Urhs' L(1/`nlags').rr quarter quartersq residr `addquarters' , fe ;
		#delimit cr
		
		quietly predict temp, ue
		quietly replace residr = temp if temp != .
		drop temp	
		
		forvalues tt = 1 (1) $Tnum{
			qui replace irfr_us_h_p`tt'    =  _b[UUp`tt']  if trend==`x'+1  
			qui replace irfr_us_h_p`tt'_se = _se[UUp`tt']  if trend==`x'+1  
		}
		
		**********************
		* GDP US
		**********************		
		quietly replace ypanel = F`x'.yy-L1.yy
		qui replace ypanel=. if dum_us!=1
		di "lag US" `x'
		
		#delimit ;
		qui xtreg ypanel `Urhs' L(1/`nlags').yy quarter quartersq residu `addquarters', fe;		
		#delimit cr
		
		quietly predict temp, ue
		quietly replace residu = temp if temp != .
		drop temp	
		
		forvalues tt = 1 (1) $Tnum{
			qui replace irfy_us_h_p`tt'    =  _b[UUp`tt']  if trend==`x'+1  
			qui replace irfy_us_h_p`tt'_se = _se[UUp`tt']  if trend==`x'+1  
		}
	
		**********************
		* GDP EME
		**********************		
		quietly replace ypanel = F`x'.yy-L1.yy
		qui replace ypanel=. if dum_eme!=1
		di "lag EME" `x'
		
		if `nlagslp' == 0 {
		qui xtscc ypanel `Urhs' quarter_c* quartersq_c*  reside `addquarters' , fe 
		} 
		if `nlagslp' > 0 {		
		qui xtscc ypanel `Urhs'  L(1/`nlagslp').yy quarter_c* quartersq_c*  reside `addquarters' , fe 		
		} 
		
		quietly predict temp, residuals
		quietly replace reside = temp if temp != .
		drop temp	
		
		forvalues tt = 1 (1) $Tnum{
			qui replace irfy_eme_h_p`tt'    =  _b[UUp`tt']  if trend==`x'+1  
			qui replace irfy_eme_h_p`tt'_se = _se[UUp`tt']  if trend==`x'+1  
		}				
		
		**********************
		* GDP AFE
		**********************		
		quietly replace ypanel = F`x'.yy-L1.yy
		qui replace ypanel=. if dum_afe!=1
		di "lag AFE" `x'
		
		if `nlagslp' == 0 {
		qui xtscc ypanel `Urhs' quarter_c* quartersq_c*  resida `addquarters'  , fe
		} 
		if `nlagslp' > 0 {
		qui xtscc ypanel `Urhs' L(1/`nlagslp').yy quarter_c* quartersq_c* resida `addquarters' , fe
		} 
		
		quietly predict temp, residuals
		quietly replace resida = temp if temp != .
		drop temp			
		
		forvalues tt = 1 (1) $Tnum{
			qui replace irfy_afe_h_p`tt'    =  _b[UUp`tt']  if trend==`x'+1  
			qui replace irfy_afe_h_p`tt'_se = _se[UUp`tt']  if trend==`x'+1  
		}
						
}	


// Compute IRFS //

*** US
forvalues tt = 1 (1) $Tnum{
	* GDP
	qui gen irfy_us_h_p`tt'_lb = irfy_us_h_p`tt' - irfy_us_h_p`tt'_se
	qui gen irfy_us_h_p`tt'_ub = irfy_us_h_p`tt' + irfy_us_h_p`tt'_se
	
	* FFR
	qui gen irfr_us_h_p`tt'_lb = irfr_us_h_p`tt' - irfr_us_h_p`tt'_se
	qui gen irfr_us_h_p`tt'_ub = irfr_us_h_p`tt' + irfr_us_h_p`tt'_se
	
	* EME
	qui gen irfy_eme_h_p`tt'_lb = irfy_eme_h_p`tt' - irfy_eme_h_p`tt'_se
	qui gen irfy_eme_h_p`tt'_ub = irfy_eme_h_p`tt' + irfy_eme_h_p`tt'_se
	
	* AFE
	qui gen irfy_afe_h_p`tt'_lb = irfy_afe_h_p`tt' - irfy_afe_h_p`tt'_se
	qui gen irfy_afe_h_p`tt'_ub = irfy_afe_h_p`tt' + irfy_afe_h_p`tt'_se
	
	
}


// Save IRFs //
if `save_dum' == 1 {
	
	* EME
	preserve
	keep if countrylong=="Mexico" & trend <= `plag' + 1	
	keep trend irfy_eme_* 
	keep if trend <= `plag' + 1
	rename trend hproj
	replace hproj  = hproj - 1
	
	outsheet using IRF_EME_ns_`sname'`addname'_samplesplit_file.csv, comma noquote replace
	restore
	
	* AFE
	preserve
	keep if countrylong=="Italy" & trend <= `plag' + 1	
	keep trend irfy_afe_* 
	keep if trend <= `plag' + 1
	rename trend hproj
	replace hproj  = hproj - 1
	
	outsheet using IRF_AFE_ns_`sname'`addname'_samplesplit_file.csv, comma noquote replace
	restore
	
	* US
	preserve
	keep if countrylong=="United States" & trend <= `plag' + 1	
	keep trend irfy_us_* irfr_us_*
	keep if trend <= `plag' + 1
	rename trend hproj
	replace hproj  = hproj - 1
	
	outsheet using IRF_US_ns_`sname'`addname'_samplesplit_file.csv, comma noquote replace
	restore

}


// PLOTS //
gen zero_line = 0*years_after		

local plotcount = 0
forvalues tt = 1 (1) $Tnum{

	* dates
	local tlb = `"T`tt'lb"'
	local tub = `"T`tt'ub"'
	
	***************
	***   EME   ***
	***************
	local name = "  $`tlb' - $`tub' "		
	
	#delimit ;
	twoway(rarea irfy_eme_h_p`tt'_ub irfy_eme_h_p`tt'_lb years_after  
	if  trend<=`plag' & countrylong=="Mexico" , color(eltblue) ) // ylabel(, ax(2) nogrid)
	
	  (line irfy_eme_h_p`tt' zero_line  years_after  // twoway(line irfy_eme_h irfy_eme_`vv'_f  `ADJnames' zero_line years_after 
	if trend<=`plag' & countrylong=="Mexico", 
	lc(dknavy black) 
	lwidth(thick thin thick thick thick thick) 
	plotr(lwidth(none))  
	yaxis(1 2) ) , 
	
	title("`name'", color(black))   // title("`ff'". EME GDP - % -- F, color(black)) 
	legend(off)
	graphr(color(white))
	xlabel(, ax(1) tp(i))  
	ylabel(, ax(1) noticks nolabels nogrid)
	ylabel(, ax(2)  nogrid)
	ylabel(, ax(1) tp(i))
	// ylabel(-1.5(.5)1.5, ax(2) tp(i)) 
	plotr(lwidth(none)) 
	
	name("Ep`tt'", replace) ;
	
	#delimit cr
	
	***************
	***   AFE   ***
	***************
	local name = " $`tlb' - $`tub' "		
	
	#delimit ;
	twoway(rarea irfy_afe_h_p`tt'_ub irfy_afe_h_p`tt'_lb years_after  
	if  trend<=`plag' & countrylong=="Italy" , color(eltblue) ) // ylabel(, ax(2) nogrid)
	
	  (line irfy_afe_h_p`tt' zero_line  years_after  // twoway(line irfy_eme_h irfy_eme_`vv'_f  `ADJnames' zero_line years_after 
	if trend<=`plag' & countrylong=="Italy", 
	lc(dknavy black) 
	lwidth(thick thin thick thick thick thick) 
	plotr(lwidth(none))  
	yaxis(1 2) ) , 
	
	title("`name'", color(black))   
	legend(off)
	graphr(color(white))
	xlabel(, ax(1) tp(i))  
	ylabel(, ax(1) noticks nolabels nogrid)
	ylabel(, ax(2)  nogrid)
	ylabel(, ax(1) tp(i))
	// ylabel(-1.5(.5)1.5, ax(2) tp(i)) 
	plotr(lwidth(none)) 
	
	name("Ap`tt'", replace) ;
	
	#delimit cr
	
	***************
	***   US   ***
	***************
	local name = " $`tlb' - $`tub' "		
	
	#delimit ;
	twoway(rarea irfy_us_h_p`tt'_ub irfy_us_h_p`tt'_lb years_after  
	if  trend<=`plag' & countrylong=="United States"  ) 
	
	  (line irfy_us_h_p`tt' zero_line  years_after  
	if trend<=`plag' & countrylong=="United States", 
	lc(red black) 
	lwidth(thick thin thick thick thick thick) 
	plotr(lwidth(none))  
	yaxis(1 2) ) , 
	
	title("`name'", color(black))   
	legend(off)
	graphr(color(white))
	xlabel(, ax(1) tp(i))  
	ylabel(, ax(1) noticks nolabels nogrid)
	ylabel(, ax(2)  nogrid)
	ylabel(, ax(1) tp(i))
	// ylabel(-1.5(.5)1.5, ax(2) tp(i)) 
	plotr(lwidth(none)) 
	
	name("USyp`tt'", replace) ;
	
	#delimit cr
	
	local name = " $`tlb' - $`tub' "		
	
	#delimit ;
	twoway(rarea irfr_us_h_p`tt'_ub irfr_us_h_p`tt'_lb years_after  
	if  trend<=`plag' & countrylong=="United States" ) 
	
	  (line irfr_us_h_p`tt' zero_line  years_after  
	if trend<=`plag' & countrylong=="United States", 
	lc(red black) 
	lwidth(thick thin thick thick thick thick) 
	plotr(lwidth(none))  
	yaxis(1 2) ) , 
	
	title("`name'", color(black))   
	legend(off)
	graphr(color(white))
	xlabel(, ax(1) tp(i))  
	ylabel(, ax(1) noticks nolabels nogrid)
	ylabel(, ax(2)  nogrid)
	ylabel(, ax(1) tp(i))
	// ylabel(-1.5(.5)1.5, ax(2) tp(i)) 
	plotr(lwidth(none)) 
	
	name("USrp`tt'", replace) ;
	
	#delimit cr

}

local A_plot   = " " 
local E_plot   = " " 
local USy_plot = " " 
local USr_plot = " " 
forvalues tt = 1 (1) $Tnum{
	local A_plot   = " `A_plot' Ap`tt'"
	local E_plot   = " `E_plot' Ep`tt'"
	local USy_plot = " `USy_plot' USyp`tt'"
	local USr_plot = " `USr_plot' USrp`tt'"
}
graph combine `A_plot', rows(1) name(AFE, replace) title(AFE: GDP Response by Period) ycommon
graph combine `E_plot', rows(1) name(EME, replace) title(EME: GDP Response by Period) ycommon

graph combine AFE EME, rows(2) name(AFEEME, replace)
window manage close graph "AFE" 
window manage close graph "EME" 
forvalues tt = 1 (1) $Tnum{
	window manage close graph "Ap`tt'" 
	window manage close graph "Ep`tt'" 
}


graph combine `USy_plot', rows(1) name(USy, replace) title(US: GDP Response by Period) ycommon
graph combine `USr_plot', rows(1) name(USr, replace) title(US: FFR Response by Period) ycommon
graph combine USy USr, rows(2) name(US, replace)
window manage close graph "USy" 
window manage close graph "USr" 
forvalues tt = 1 (1) $Tnum{
	window manage close graph "USyp`tt'" 
	window manage close graph "USrp`tt'" 
}



graph combine USy USr AFE EME, rows(2) name(USAFEEME, replace)		
gr export FIGURE11.pdf, mag(100) replace name(USAFEEME) 

