clear
set more off

local path    = "U:\tf\Matteo\Rshock\Replication"	    // where files for regressions are



cls
cd `path'

adopath ++ `path'\ADOFILES
set scrollbufsize 300000

use PANEL_DATASET, clear

set scheme s1color



local smpl1 "1965q1"
local smpl2 "2016q2"


local ss_val = 2 // State dependent regressions = ss_val = 1 (otherwise 2)


local ii_val= 1 // 1-Monetary Shock; 2-Demand Shocks
local nlags=4	 // Lags of LHS variables as controls in shock estimation 
local nlagslp=2	 // Lags of LHS variables as controls in local projection estimation
local kk = 1     // Convexification factor for index
local sh = 1 	 // IRF on sh standard deviations
local lagsvv = 1 // Lags of conditioning variable in interaction term
local plag = 2  // Periods for projection





if `ii_val' == 1{ 
local sname = "rshock" 
}
if `ii_val' == 2{ 
local sname = "yshock" 
}


* Fragility measures to use
* 1 - High Inflation: BAD
* 2 - High CA: GOOD
* 3 - High Reserves: GOOD
* 4 - High External Debt: BAD
* 5 - High GDP PC relative to average: GOOD
* 6 - Fast Growth
* 7 - Fixed Exchange Rate
* 8 - Trade with US
* 9 - Trade with RoW
* 10 - External Debt xres
* 11 - Change in External Debt

sum vv*


local data_fill = 1

if `data_fill'==0 {
replace vvf1 = vv1
replace vvf2 = vv2
replace vvf3 = vv3
replace vvf4 = vv4
replace vvf5 = vv5
replace vvf6 = vv6
replace vvf7 = vv7
replace vvf8 = vv8
replace vvf9 = vv9
replace vvf10 = vv10
replace vvf11 = vv11
}




local frag_list "vvf7 vvf8 vvfindex "

local pca_list "vvf1 vvf2 vvf3 vvf10"  



gen vvfindex = .


replace vvfindex  =  0.5*(vvf1 + vvf2 + vvf3 + vvf10)






#delimit ;
global vvf1_pcl  = 50; global vvf1_pcu  = 95; global vvf1_kk  = `kk';  
global vvf2_pcl  = 50; global vvf2_pcu  = 95; global vvf2_kk  = `kk';  
global vvf3_pcl  = 50; global vvf3_pcu  = 95; global vvf3_kk  = `kk';   
global vvf4_pcl  = 50; global vvf4_pcu  = 95; global vvf4_kk  = `kk';  
global vvf5_pcl  = 50; global vvf5_pcu  = 95; global vvf5_kk  = `kk';  
global vvf6_pcl  = 50; global vvf6_pcu  = 95; global vvf6_kk  = `kk';  
global vvf7_pcl  = 50; global vvf7_pcu  = 95; global vvf7_kk  = `kk';  
global vvf8_pcl  = 50; global vvf8_pcu  = 95; global vvf8_kk  = `kk';  
global vvf9_pcl  = 50; global vvf9_pcu  = 95; global vvf9_kk  = `kk';  
global vvf10_pcl = 50; global vvf10_pcu = 95; global vvf10_kk = `kk';  
global vvf11_pcl = 50; global vvf11_pcu = 95; global vvf11_kk = `kk';  

global vvfindex_pcl = 50; global vvfindex_pcu = 95; global vvfindex_kk = `kk';  

global vvf7_pcz  = 15;
# delimit cr






local frag_controls = "UU`ii_val'" 

if `ss_val' == 1{
	foreach vv of local frag_list {  
		
		local newvar = "`frag_controls' UU`vv'_f"   
		local frag_controls = "`newvar'"		
	}
}


di "`frag_controls'"





levelsof quarter , c l(datelist)
foreach qq of local datelist {
	local name = string(`qq', "%tq")
	gen dum`name' = cond(quarter == `qq',1,0)
}





drop US_r_shock*

gen years_after = (trend-1)/4
label var years_after "Year Since Shock"


***************************************************
* PART 1: CONSTRUCT US MONETARY AND DEMAND SHOCKS
***************************************************

xtset country quarter, quarterly 
gen qq = quarter(dofq(quarter)) // qofd(quarter)
gen yy = year(dofq(quarter)) 
gen yq = yy + (qq-1)/4

drop yy qq

order countrylong quarter year yq 

preserve

keep if countrylong=="United States"



#delimit ;
local shockregression_controls_r = 
"L(1/`nlags').US_r 
L(0/`nlags').US_y 
L(0/`nlags').US_dp 
L(0/`nlags').US_spread 
L(0/`nlags').ln_gdp_for 
quarter quartersq" ;
#delimit cr 

regress US_r `shockregression_controls_r'
predict US_r_fit
predict UU1, residuals
gen UUr = UU1


	
tsline UUr if tin(1965q1,2016q2), name(US_SHOCKS, replace) 
gr export FIGURE3.pdf, mag(100) replace name(US_SHOCKS, replace) replace



