clear
set more off

local path    = "U:\tf\Matteo\Rshock\Replication"	    // where files for regressions are

cls
cd `path'

adopath ++ `path'\ADOFILES
set scrollbufsize 300000

use PANEL_DATASET, clear

set scheme s1color



local smpl1 "1965q1"
local smpl2 "2016q2"



local ii_val= 1 // 1-Monetary Shock; 2-Demand Shocks
local nlags=4	 // Lags of LHS variables as controls in shock estimation 
local nlagslp=2	 // Lags of LHS variables as controls in local projection estimation
local kk = 1     // Convexification factor for index
local sh = 1 	 // IRF on sh standard deviations
local lagsvv = 1 // Lags of conditioning variable in interaction term
local plag = 2  // Periods for projection
local ss_val = 1 // State dependent regressions = ss_val = 1 (otherwise 2)



if `ii_val' == 1{ 
local sname = "rshock" 
}
if `ii_val' == 2{ 
local sname = "yshock" 
}


* Fragility measures to use
* 1 - High Inflation: BAD
* 2 - High CA: GOOD
* 3 - High Reserves: GOOD
* 4 - High External Debt: BAD
* 5 - High GDP PC relative to average: GOOD
* 6 - Fast Growth
* 7 - Fixed Exchange Rate
* 8 - Trade with US
* 9 - Trade with RoW
* 10 - External Debt xres
* 11 - Change in External Debt

sum vv*


local data_fill = 1

if `data_fill'==0 {
replace vvf1 = vv1
replace vvf2 = vv2
replace vvf3 = vv3
replace vvf4 = vv4
replace vvf5 = vv5
replace vvf6 = vv6
replace vvf7 = vv7
replace vvf8 = vv8
replace vvf9 = vv9
replace vvf10 = vv10
replace vvf11 = vv11
}




local frag_list "vvf7 vvf8 vvf2 vvf3 vvf1 vvf10"  // vvf5
//scatter vvfindex vvf1 if tin(2015q4,2015q4), mlabel(countryname)

gen vvfindex = .

#delimit ;
global vvf1_pcl  = 50; global vvf1_pcu  = 95; global vvf1_kk  = `kk';  
global vvf2_pcl  = 50; global vvf2_pcu  = 95; global vvf2_kk  = `kk';  
global vvf3_pcl  = 50; global vvf3_pcu  = 95; global vvf3_kk  = `kk';   
global vvf4_pcl  = 50; global vvf4_pcu  = 95; global vvf4_kk  = `kk';  
global vvf5_pcl  = 50; global vvf5_pcu  = 95; global vvf5_kk  = `kk';  
global vvf6_pcl  = 50; global vvf6_pcu  = 95; global vvf6_kk  = `kk';  
global vvf7_pcl  = 50; global vvf7_pcu  = 95; global vvf7_kk  = `kk';  
global vvf8_pcl  = 50; global vvf8_pcu  = 95; global vvf8_kk  = `kk';  
global vvf9_pcl  = 50; global vvf9_pcu  = 95; global vvf9_kk  = `kk';  
global vvf10_pcl = 50; global vvf10_pcu = 95; global vvf10_kk = `kk';  
global vvf11_pcl = 50; global vvf11_pcu = 95; global vvf11_kk = `kk';  

global vvfindex_pcl = 50; global vvfindex_pcu = 95; global vvfindex_kk = `kk';  

global vvf7_pcz  = 15;
# delimit cr







local frag_controls = "UU`ii_val'" // " " // "UU`ii_val'"

if `ss_val' == 1{
	foreach vv of local frag_list {  // foreach xx in 1 2 3 4 5 { 	
		
		local newvar = "`frag_controls' UU`vv'_f"   // "`frag_controls' UUF`xx'_h UUF`xx'_f"  
		local frag_controls = "`newvar'"		
	}
}
/*
if `ss_val' == 2{
	foreach vv of local frag_list {  // foreach xx in 1 2 3 4 5 { 			
		local newvar = "`frag_controls' UU`vv'"  
		local frag_controls = "`newvar'"		
	}
}
*/
di "`frag_controls'"





levelsof quarter , c l(datelist)
foreach qq of local datelist {
	local name = string(`qq', "%tq")
	gen dum`name' = cond(quarter == `qq',1,0)
}



*replace US_y=. if tin(1960q1,1983q4)
*replace US_y=. if tin(2008q1,2016q2)


drop US_r_shock*

gen years_after = (trend-1)/4
label var years_after "Year Since Shock"


***************************************************
* PART 1: CONSTRUCT US MONETARY AND DEMAND SHOCKS
***************************************************

xtset country quarter, quarterly 
gen qq = quarter(dofq(quarter)) // qofd(quarter)
gen yy = year(dofq(quarter)) 
gen yq = yy + (qq-1)/4

drop yy qq

order countrylong quarter year yq 

preserve

keep if countrylong=="United States"


// keep if yq <= 1985.75
// keep if yq >  1985.75
// keep if yq >= 1984

#delimit ;
local shockregression_controls_r = 
"L(1/`nlags').US_r 
L(0/`nlags').US_y 
L(0/`nlags').US_dp 
L(0/`nlags').US_spread 
L(0/`nlags').ln_gdp_for 
quarter quartersq" ;
#delimit cr 

regress US_r `shockregression_controls_r'
predict US_r_fit
predict UU1, residuals
gen UUr = UU1


//L(1/`nlags').ln_gdp_for 

#delimit ;
local shockregression_controls_y = 
"L(1/`nlags').US_r 
L(1/`nlags').US_y 
L(1/`nlags').US_dp 
L(1/`nlags').ln_gdp_for 
L(1/`nlags').US_spread 
quarter quartersq" ;
#delimit cr 

regress US_y `shockregression_controls_y'
predict US_y_fit
predict UU2, residuals
gen UUy = UU2


keep quarter yq US* UU*
tempfile US_file2
save `US_file2'



restore

merge m:1 quarter using `US_file2', nogen
xtset country quarter, quarterly 


sort country quarter
gen yy  = ln_gdp
gen yyr = ln_gdp_c
gen rr  = US_r
gen xr  = US_xr

tssmooth ma UU1_ma4 = UU1, window(3 1 0)
tssmooth ma UU2_ma4 = UU2, window(3 1 0)

label var UU1_ma4 "Monetary Shock, 4 qtr MA"
label var UU2_ma4 "Demand Shock, 4 qtr MA"
label var rr "FFR"
label var d4_ln_gdp_us "US GDP growth yoy, %"





foreach mm in 1 4 6 8 10 12 16 18{

gen f`mm'yy = F`mm'.yy-L1.yy 
egen f`mm'yy_m = mean(f`mm'yy), by(country) 
gen f`mm'yyd = f`mm'yy - f1yy_m 
gen f`mm'xr = F`mm'.xr-L1.xr 
egen f`mm'xr_m = mean(f`mm'xr), by(country) 
gen f`mm'xrd = f`mm'xr - f1xr_m 

}








gen ypanel = .
gen irfy_f = .
gen irfy_f_se = .
gen irfy_h = .
gen irfy_h_se = .


label var trend "Quarters"

**********************************************
* 1.5- DECLARE SPACE FOR IRFS COMPUTATIONS
**********************************************

gen residu = 0
gen resida = 0
gen reside = 0
gen residr = 0
gen residx = 0
gen residm = 0

qui gen irfy_eme_h    = .  
qui gen irfy_eme_h_se = .  
qui gen irfy_afe_h    = .  
qui gen irfy_afe_h_se = .  
qui gen irfy_us_h    = .  
qui gen irfy_us_h_se = .  
qui gen irfr_us_h    = .  
qui gen irfr_us_h_se = .  

if `ss_val' == 1{
	foreach vv of local frag_list {	
	
		// EME GDP		
		qui gen irfy_eme_`vv'_f    = .  
		qui gen irfy_eme_`vv'_f_se = .  
		
		if "`vv'" == "vvf7"{
		qui gen irfy_eme_LBADJ`vv'_f = .
		}
		
		// AFE GDP
		qui gen irfy_afe_`vv'_f    = .  
		qui gen irfy_afe_`vv'_f_se = .  
		
		if "`vv'" == "vvf7"{
		qui gen irfy_afe_LBADJ`vv'_f = .
		}
		
	}
}	



**************************************
* 2-  PANEL REGRESSIONS BEGINS HERE 
**************************************

sort country quarter


/////////////////////////////////////////////
// CREATE LHS VARIABLE
/////////////////////////////////////////////
// NOTE: NAIVE IS WHEN WE ASSUME THAT QUARTERLY GDP GROWTH EQUALS ANNNUAL GDP GROWTH FOR THE PERIOD
// IN WHICH WE HAVE ONLY ANNUAL DATA

// local addsname = "NAIVE" // "RR" "ZLB" "INT" "NAIVE" "NBR"

// replace UU1 = RRSHOCK

// replace yy=. if tin(2007q1,2016q4)

// replace yy = ln_gdp_old 

// replace yy = ln_gdp_naive

// replace UU1 = . if tin(1979q1,1982q4)




if `ii_val' == 1 {
		local addquarters = "dum1981q4"
		local titler "1. Fed Funds Rate - ppt"
		local titley "2. U.S. GDP - %"
}
if `ii_val' == 2 {
		local addquarters = " "
		local titler = "2. Fed Funds Rate - ppt"
		local titley = "1. U.S. GDP - %"
}

*gen aux`vv' = exp(`kk' *  I`vv' )/(1+exp(`kk' * I`vv' ))

* RANDOMLY BREAK COUNTRIES DOWN
* replace vv4 = rnormal()*0.1

local orth_list " "
pause on

if `ss_val' == 1 {
foreach vv of local frag_list {
	
	gen `vv'_dm = .	
	***AFE		
	qui sum `vv' if dum_afe == 1, detail	
	replace `vv'_dm = (`vv' - r(p50))/r(sd) if dum_afe == 1	
	*** EME
	qui sum `vv' if dum_eme == 1, detail
	replace `vv'_dm = (`vv' - r(p50))/r(sd) if dum_eme == 1

	local kk = `"$`vv'_kk"'

	gen aux`vv' = exp(`kk' * `vv'_dm )/(1+exp(`kk' * `vv'_dm )) // gen aux`vv' = exp(`kk' * `vv' )/(1+exp(`kk' * `vv' ))

	replace aux`vv' = 0 if countrylong == "United States"
	
	gen     F`vv' = .
	replace F`vv' = L`lagsvv'.aux`vv'		
	// replace F`vv' = L`lagsvv'.`vv'_dm		
	
	local vax_l = `"$`vv'_pcl"'
	local vax_u = `"$`vv'_pcu"'	
	
	egen pcl1 = pctile(F`vv') if dum_afe==1, p(`vax_l')  
	egen pcl2 = pctile(F`vv') if dum_eme==1, p(`vax_l')
	
	egen pcu1 = pctile(F`vv') if dum_afe==1, p(`vax_u')   // egen pcu1 = sd(F`vv') if dum_afe==1
	egen pcu2 = pctile(F`vv') if dum_eme==1, p(`vax_u')   // egen pcu2 = sd(F`vv') if dum_eme==1
	
	gen     pclx = pcl1 if dum_afe==1
	replace pclx = pcl2 if dum_eme==1	
	gen     pcux = pcu1 if dum_afe==1
	replace pcux = pcu2 if dum_eme==1	
	
	gen UU`vv'_f  = UU`ii_val' * (F`vv'-pclx)/(pcux-pclx)  // 	gen UU`vv'_f  = UU`ii_val' * (F`vv'-pclx)/(pcux) 
	
	if "`vv'" == "vvf7"{
		local vax_z = `"$`vv'_pcz"'	
		egen pcz1 = pctile(F`vv') if dum_afe==1, p(`vax_z')  
		egen pcz2 = pctile(F`vv') if dum_eme==1, p(`vax_z')  
		
		gen     LBAX = (pcz1 - pcl1)/(pcu1 - pcl1) if dum_afe == 1
		replace LBAX = (pcz2 - pcl2)/(pcu2 - pcl2) if dum_eme == 1
		
		qui sum LBAX if dum_afe == 1		
		global LBADJ_AFE = r(mean)
		qui sum LBAX if dum_eme == 1		
		global LBADJ_EME = r(mean)
	}
	
	xtreg UU`vv'_f UU`ii_val' `orth_list' L(1/`nlags').yy quarter quartersq if dum_afe==1, fe
	quietly predict temp if dum_afe==1, ue
	quietly replace UU`vv'_f = temp if dum_afe==1
	drop temp	
	
	xtreg UU`vv'_f UU`ii_val' `orth_list' L(1/`nlags').yy quarter quartersq if dum_eme==1, fe
	quietly predict temp if dum_eme==1, ue
	quietly replace UU`vv'_f = temp if dum_eme==1
	drop temp	
	
	local orth_list = "`orth_list' UU`vv'_f "	
	
	drop pclx pcl1 pcl2 pcux pcu1 pcu2 
}
}

di $LBADJ_EME


if `ss_val' == 1 {

 tabstat vv* , statistics(p10 p25 p50 p75 p90 sd mean)
 tabstat vvf* if dum_eme==1, statistics(p10 p25 p50 p75 p90 p99 sd mean)
 tabstat index*_std if dum_eme==1, statistics(p10 p25 p50 p75 p90 sd mean)
 tabstat auxvvf* if dum_eme==1, statistics(p10 p25 p50 p75 p90 p95 p99 sd mean)

 tabstat vvf* if dum_afe==1, statistics(p10 p25 p50 p75 p90 p99 sd mean)
 tabstat index*_std if dum_afe==1, statistics(p10 p25 p50 p75 p90 sd mean)
 tabstat auxvvf* if dum_afe==1, statistics(p10 p25 p50 p75 p90 p95 p99 sd mean)
 ** di "`frag_controls'"
}

// save data for plots
preserve
keep auxv* countrylong quarter year yq countryname country
save datasetforplots, replace
restore



///xtreg f12yyd UUr UUvvf1_f UUvvf2_f UUvvf6_f UUvvf9_f if dum_eme==1, fe
///xtreg f12yyd UUr UUvvf1_f UUvvf2_f UUvvf6_f UUvvf9_f if dum_afe==1, fe



// Regression step //
forvalues x = 0/`plag' {
		

		**********************
		* R US
		**********************		
		quietly replace ypanel = F`x'.rr-L1.rr
		qui replace ypanel=. if dum_us!=1
		di "lag US" `x'
		
		#delimit ;
		qui xtreg ypanel UU`ii_val' L(1/`nlags').rr quarter quartersq residr `addquarters' , fe ;
		#delimit cr
		
		quietly predict temp, ue
		quietly replace residr = temp if temp != .
		drop temp	
		
    	qui replace irfr_us_h    = _b[UU`ii_val']   if trend==`x'+1  
	    qui replace irfr_us_h_se = _se[UU`ii_val']  if trend==`x'+1  
		
		
		**********************
		* GDP US
		**********************		
		quietly replace ypanel = F`x'.yy-L1.yy
		qui replace ypanel=. if dum_us!=1
		di "lag US" `x'
		
		#delimit ;
		qui xtreg ypanel UU`ii_val' L(1/`nlags').yy quarter quartersq residu `addquarters', fe;		
		#delimit cr
		
		quietly predict temp, ue
		quietly replace residu = temp if temp != .
		drop temp	
		
    	qui replace irfy_us_h    = _b[UU`ii_val']   if trend==`x'+1  
	    qui replace irfy_us_h_se = _se[UU`ii_val']  if trend==`x'+1  
		
	
		**********************
		* GDP EME
		**********************		
		quietly replace ypanel = F`x'.yy-L1.yy
		qui replace ypanel=. if dum_eme!=1
		di "lag EME" `x'
		
		if `nlagslp' == 0 {
		qui xtscc ypanel `frag_controls' quarter_c* quartersq_c*  reside `addquarters' , fe 
		} 
		if `nlagslp' > 0 {
		qui xtscc ypanel `frag_controls' L(1/`nlagslp').yy quarter_c* quartersq_c* reside `addquarters' , fe 
		} 

		
		quietly predict temp, residuals
		quietly replace reside = temp if temp != .
		drop temp	
		
		if `ss_val' == 1{
			foreach vv of local frag_list {
				qui replace irfy_eme_h    = _b[UU`ii_val']  if trend==`x'+1  
				qui replace irfy_eme_h_se = _se[UU`ii_val'] if trend==`x'+1 
				
				qui lincom UU`ii_val' + `sh'*UU`vv'_f
				local cc = r(estimate) 
				local ss = r(se)	  
				qui replace irfy_eme_`vv'_f    = `cc'  if trend==`x'+1  
				qui replace irfy_eme_`vv'_f_se = `ss'  if trend==`x'+1 
				
				if "`vv'" == "vvf7"{
				qui lincom UU`ii_val' + $LBADJ_EME * UU`vv'_f
				local cc = r(estimate) 
				local ss = r(se)	   
				qui replace irfy_eme_LBADJ`vv'_f    = `cc' if trend==`x'+1  
				}
			}
			
		}
		if `ss_val' == 2{
			qui replace irfy_eme_h    = _b[UU`ii_val']  if trend==`x'+1  
			qui replace irfy_eme_h_se = _se[UU`ii_val'] if trend==`x'+1 
			
		}
		
		
		**********************
		* GDP AFE
		**********************		
		quietly replace ypanel = F`x'.yy-L1.yy
		qui replace ypanel=. if dum_afe!=1
		di "lag AFE" `x'
		
		if `nlagslp' == 0 {
		qui xtscc ypanel `frag_controls' quarter_c* quartersq_c*  resida `addquarters' if , fe 
		} 
		if `nlagslp' > 0 {
		qui xtscc ypanel `frag_controls' L(1/`nlagslp').yy quarter_c* quartersq_c* resida `addquarters' , fe 		
		} 
		
		quietly predict temp, residuals
		quietly replace resida = temp if temp != .
		drop temp			
		
		if `ss_val' == 1{
			foreach vv of local frag_list {
				qui replace irfy_afe_h    = _b[UU`ii_val']  if trend==`x'+1  
				qui replace irfy_afe_h_se = _se[UU`ii_val'] if trend==`x'+1  
				
				qui lincom UU`ii_val' + `sh'*UU`vv'_f
				local cc = r(estimate) 
				local ss = r(se)      
				qui replace irfy_afe_`vv'_f    = `cc'  if trend==`x'+1  
				qui replace irfy_afe_`vv'_f_se = `ss'  if trend==`x'+1  
				
				if "`vv'" == "vvf7"{
				qui lincom UU`ii_val' + $LBADJ_AFE * UU`vv'_f
				local cc = r(estimate) 
				local ss = r(se)	   
				qui replace irfy_afe_LBADJ`vv'_f    = `cc' if trend==`x'+1  				
				}
			}
		}
		if `ss_val' == 2{
			qui replace irfy_afe_h    = _b[UU`ii_val']  if trend==`x'+1  
			qui replace irfy_afe_h_se = _se[UU`ii_val'] if trend==`x'+1 
			
		}				
}		


// Compute IRFS //

*** US
* GDP
qui gen irfy_us_h_lb = irfy_us_h - irfy_us_h_se
qui gen irfy_us_h_ub = irfy_us_h + irfy_us_h_se
* FFR
qui gen irfr_us_h_lb = irfr_us_h - irfr_us_h_se
qui gen irfr_us_h_ub = irfr_us_h + irfr_us_h_se

if `ss_val' == 1{
	
	* EME
	qui gen irfy_eme_h_lb = irfy_eme_h - irfy_eme_h_se
	qui gen irfy_eme_h_ub = irfy_eme_h + irfy_eme_h_se

	foreach vv of local frag_list {
	qui gen irfy_eme_`vv'_f_lb = irfy_eme_`vv'_f - irfy_eme_`vv'_f_se
	qui gen irfy_eme_`vv'_f_ub = irfy_eme_`vv'_f + irfy_eme_`vv'_f_se	
	}	
	
	
	* AFE
	qui gen irfy_afe_h_lb = irfy_afe_h - irfy_afe_h_se
	qui gen irfy_afe_h_ub = irfy_afe_h + irfy_afe_h_se

	foreach vv of local frag_list {
	qui gen irfy_afe_`vv'_f_lb = irfy_afe_`vv'_f - irfy_afe_`vv'_f_se
	qui gen irfy_afe_`vv'_f_ub = irfy_afe_`vv'_f + irfy_afe_`vv'_f_se
	}
	
}
if `ss_val' == 2{
	
	* EME
	qui gen irfy_eme_h_lb = irfy_eme_h - irfy_eme_h_se
	qui gen irfy_eme_h_ub = irfy_eme_h + irfy_eme_h_se
	

	* AFE
	qui gen irfy_afe_h_lb = irfy_afe_h - irfy_afe_h_se
	qui gen irfy_afe_h_ub = irfy_afe_h + irfy_afe_h_se	
	
}



if `ii_val' == 1{
	global adj = irfr_us_h[1]	
	
	local varaux " irfr_us_h  irfr_us_h_lb  irfr_us_h_ub irfy_us_h  irfy_us_h_lb  irfy_us_h_ub"
	local varaux " `varaux' irfy_afe_h irfy_afe_h_lb irfy_afe_h_ub "
	local varaux " `varaux' irfy_eme_h irfy_eme_h_lb irfy_eme_h_ub "				
	
	if `ss_val' == 1{
	foreach vv of local frag_list {
		local varaux = " `varaux' irfy_afe_`vv'_f irfy_afe_`vv'_f_lb irfy_afe_`vv'_f_ub "
		local varaux = " `varaux' irfy_eme_`vv'_f irfy_eme_`vv'_f_lb irfy_eme_`vv'_f_ub "
	}
	}
	
	foreach vv of local varaux{		
		replace `vv' = `vv' / $adj		
	}
}









// PLOTS //

label var vvf1 "Inflation"
label var vvf2 "Current Account"
label var vvf3 "Reserves"
label var vvf4 "External Debt"
label var vvf5 "Cyclical GDP"
label var vvf6 "Cyclical GDP"
label var vvf7 "Dollar Peg"
label var vvf8 "Trade with U.S."
label var vvf9 "Trade with ROW"
label var vvf10 "External Debt ex res"
label var vvf11 "Change in External Debt"
label var vvfindex "Vulnerability Index "


gen zero_line = 0*years_after		

foreach vv of local frag_list {

	if `ss_val' == 1{	
	
	* EME
	local name : variable label `vv' 
	if `"`vv'"' == "" local name "`vv'" 	// local name =  "`vv'. EME GDP - % "
	
	#delimit ;
	drarea irfy_eme_h_ub irfy_eme_h_lb  irfy_eme_`vv'_f_ub irfy_eme_`vv'_f_lb years_after 
	if  trend<=`plag' & countrylong=="Mexico", 
	ylabel(, ax(2) nogrid)
	
	twoway(line irfy_eme_h irfy_eme_`vv'_f zero_line years_after 
	if trend<=`plag' & countrylong=="Mexico", 
	lc(dknavy cranberry black)
	lwidth(thick thick thin) 
	plotr(lwidth(none))  
	yaxis(1 2)) 
	
	plotr(lwidth(none)) 
	color(eltblue erose) 	
	title("`name'", color(black))  // title("`ff'". EME GDP - % -- F, color(black)) 
	legend(order(1 "Median" 2 "High") ring(0) position(8) rows(2) region(color(none))) 
	graphr(color(white))
	ysc(r(-1.5 0.5))
	
	xlabel(, ax(1) tp(i))  
	ylabel(, ax(1) noticks nolabels nogrid)
	ylabel(, ax(2)  nogrid)
	ylabel(, ax(1) tp(i))
	// ylabel(-1.5(.5)1.5, ax(2) tp(i)) 
	plotr(lwidth(none)) 
	
	name("E`vv'", replace) ;
	
	#delimit cr
	
	* AFE
	local name : variable label `vv' 
	if `"`vv'"' == "" local name "`vv'" 	// local name =  "`vv'. AFE GDP - % "
	
	#delimit ;
	drarea irfy_afe_h_ub irfy_afe_h_lb  irfy_afe_`vv'_f_ub irfy_afe_`vv'_f_lb years_after 
	if  trend<=`plag' & countrylong=="Italy", 
	ylabel(, ax(2) nogrid)
	
	twoway(line irfy_afe_h irfy_afe_`vv'_f zero_line years_after 
	if trend<=`plag' & countrylong=="Italy", 
	lc(dknavy cranberry black)
	lwidth(thick thick thin) 
	plotr(lwidth(none))  
	yaxis(1 2)) 
	
	plotr(lwidth(none)) 
	color(eltblue erose) 	
	title("`name'", color(black))  // title("`ff'". EME GDP - % -- F, color(black)) 
	legend(order(1 "Median" 2 "High") ring(0) position(8) rows(2) region(color(none))) 
	graphr(color(white))
	ysc(r(-1.5 0.5))
	
	xlabel(, ax(1) tp(i))  
	ylabel(, ax(1) noticks nolabels nogrid)
	ylabel(, ax(2)  nogrid)
	ylabel(, ax(1) tp(i))
	// ylabel(-1.5(.5)1.5, ax(2) tp(i)) 
	plotr(lwidth(none)) 
	
	name("A`vv'", replace) ;
	
	#delimit cr
	
	}
}

if `ss_val' == 2{

	* US
	local name =  " US GDP - % "
	#delimit ;
	twoway (rarea irfy_us_h_ub irfy_us_h_lb years_after 
	if  trend<=`plag' & countrylong=="United States"	 )  // ylabel(, ax(2) nogrid)
	
	  (line irfy_us_h zero_line years_after 
	if trend<=`plag' & countrylong=="United States", 
	lc(cranberry black)
	lwidth(thick thin) 
	plotr(lwidth(none))  
	yaxis(1 2) 	),
	legend(off) 
	title(" `name' ", color(black) ) // title("`ff'". EME GDP - % -- F, color(black)) 	 
	graphr(color(white))
	xlabel(, ax(1) tp(i))  
	ylabel(, ax(1) noticks nolabels nogrid)
	ylabel(, ax(2)  nogrid)
	ylabel(, ax(1) tp(i))
	plotr(lwidth(none)) 	
	name("US", replace) ;
	
	#delimit cr
	
	
	
	* US R
	local name =  " FFR - % "
	#delimit ;
	twoway (rarea irfr_us_h_ub irfr_us_h_lb years_after 
	if  trend<=`plag' & countrylong=="United States"	 )  // ylabel(, ax(2) nogrid)
	
	  (line irfr_us_h zero_line years_after 
	if trend<=`plag' & countrylong=="United States", 
	lc(cranberry black)
	lwidth(thick thin) 
	plotr(lwidth(none))  
	yaxis(1 2) 	),
	legend(off) 
	title(" `name' ", color(black) ) // title("`ff'". EME GDP - % -- F, color(black)) 	 
	graphr(color(white))
	xlabel(, ax(1) tp(i))  
	ylabel(, ax(1) noticks nolabels nogrid)
	ylabel(, ax(2)  nogrid)
	ylabel(, ax(1) tp(i))
	plotr(lwidth(none)) 	
	name("R", replace) ;
	
	#delimit cr	
	

	* EME
	local name =  " EME GDP - % "
	#delimit ;
	twoway (rarea irfy_eme_h_ub irfy_eme_h_lb years_after 
	if  trend<=`plag' & countrylong=="Mexico"	 )  // ylabel(, ax(2) nogrid)
	
	  (line irfy_eme_h zero_line years_after 
	if trend<=`plag' & countrylong=="Mexico", 
	lc(cranberry black)
	lwidth(thick thin) 
	plotr(lwidth(none))  
	yaxis(1 2) 	),
	legend(off) 
	title(" `name' ", color(black) ) // title("`ff'". EME GDP - % -- F, color(black)) 	 
	graphr(color(white))
	ysc(r(-2.5 1)) 
	xlabel(, ax(1) tp(i))  
	ylabel(, ax(1) noticks nolabels nogrid)
	ylabel(, ax(2)  nogrid)
	ylabel(, ax(1) tp(i))
	ylabel(-1.5(.5)0.5, ax(2) tp(i)) 
	plotr(lwidth(none)) 	
	name("EME", replace) ;
	
	#delimit cr
	
	* AFE
	local name =  " AFE GDP - % "
	#delimit ;
	twoway (rarea irfy_afe_h_ub irfy_afe_h_lb years_after 
	if  trend<=`plag' & countrylong=="Italy"	 )  // ylabel(, ax(2) nogrid)
	
	    (line irfy_afe_h zero_line years_after 
	if trend<=`plag' & countrylong=="Italy", 
	lc(cranberry black)
	lwidth(thick thin) 
	plotr(lwidth(none))  
	yaxis(1 2) 	),
	legend(off) 
	title(" `name' ", color(black) ) 
	graphr(color(white))
	ysc(r(-1.5 1)) 
	xlabel(, ax(1) tp(i))  
	ylabel(, ax(1) noticks nolabels nogrid)
	ylabel(, ax(2)  nogrid)
	ylabel(, ax(1) tp(i))
	ylabel(-1.5(.5)0.5, ax(2) tp(i)) 
	plotr(lwidth(none)) 	
	name("AFE", replace) ;
	
	#delimit cr	
	
	graph combine US R AFE EME, rows(1)		
	
}


if `ss_val' == 1{

local A_plot = " " 
local E_plot = " " 
foreach vv of local frag_list {  // foreach xx in 1 2 3 4 5 { 	
		
		local newvar = "`A_plot' A`vv'"   // "`frag_controls' UUF`xx'_h UUF`xx'_f"  
		local A_plot = "`newvar'"		
		
		local newvar = "`E_plot' E`vv'"   // "`frag_controls' UUF`xx'_h UUF`xx'_f"  
		local E_plot = "`newvar'"		
}

graph combine `A_plot', rows(1) name(AFE, replace) title(AFE: GDP Response by Index) ycommon
graph combine `E_plot', rows(1) name(EME, replace) title(EME: GDP Response by Index) ycommon


foreach vv of local frag_list {
	window manage close graph "A`vv'" 
	window manage close graph "E`vv'" 
}


 // scatter irfr_us_h irfy_us_h irfy_eme_h irfy_afe_h trend if dum_us==1 & trend<20, c(l l l l l)
graph combine AFE EME, rows(2) name(AFEEME, replace)
window manage close graph "AFE" 
window manage close graph "EME" 

* US
local name =  " US GDP - % "
#delimit ;
twoway (rarea irfy_us_h_ub irfy_us_h_lb years_after 
if  trend<=`plag' & countrylong=="United States"	 )  // ylabel(, ax(2) nogrid)
	
 (line irfy_us_h zero_line years_after 
if trend<=`plag' & countrylong=="United States", 
lc(cranberry black)
lwidth(thick thin) 
plotr(lwidth(none))  
yaxis(1 2) 	),
legend(off) 
title(" `name' ", color(black) ) // title("`ff'". EME GDP - % -- F, color(black)) 	 
graphr(color(white))
xlabel(, ax(1) tp(i))  
ylabel(, ax(1) noticks nolabels nogrid)
ylabel(, ax(2)  nogrid)
ylabel(, ax(1) tp(i))
plotr(lwidth(none)) 	
name("US", replace) ;
	
#delimit cr
	
	
	
* US R
local name =  " FFR - % "
#delimit ;
twoway (rarea irfr_us_h_ub irfr_us_h_lb years_after 
if  trend<=`plag' & countrylong=="United States"	 )  // ylabel(, ax(2) nogrid)
	
 (line irfr_us_h zero_line years_after 
if trend<=`plag' & countrylong=="United States", 
lc(cranberry black)
lwidth(thick thin) 
plotr(lwidth(none))  
yaxis(1 2) 	),
legend(off) 
title(" `name' ", color(black) ) // title("`ff'". EME GDP - % -- F, color(black)) 	 
graphr(color(white))
xlabel(, ax(1) tp(i))  
ylabel(, ax(1) noticks nolabels nogrid)
ylabel(, ax(2)  nogrid)
ylabel(, ax(1) tp(i))
plotr(lwidth(none)) 	
name("R", replace) ;
	

local spdfname "vars"
local pcaname "pc"
#delimit cr

foreach vv of local frag_list {
	local spdfname = "`spdfname'_`vv'"
}
foreach vv of local pca_list {
	local pcaname = "`pcaname'_`vv'"
}


}







gr export FIGURE7.pdf, mag(100) replace name(AFEEME) 
